/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import java.lang.reflect.Method;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.api.IClassTest;



/**
 * This type describes a simple invocation test.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class ArrayInvokationTest implements IClassTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(ArrayInvokationTest.class);
    private final Object                  testedObject;
    private final int                     excludeMethodsModifiers;
    
    
    
    /**
     * Builds a simple invocation test .
     * 
     * @param _testedObject
     *            the tested class.
     * @param _excludeMethodsModifiers
     */
    public ArrayInvokationTest(final Object _testedObject, final int _excludeMethodsModifiers) {
    
    
        super();
        this.testedObject = _testedObject;
        this.excludeMethodsModifiers = _excludeMethodsModifiers;
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.IClassTest#performTest()
     */
    public void performTest() throws WeakImplementationException {
    
    
        LOGGER.debug("Processing detection of out of bounds exception");
        for (final Method method : Utils.getMethods(this.testedObject, this.excludeMethodsModifiers)) {
            
            final Class<?>[] parameterTypes = method.getParameterTypes();
            final int length = parameterTypes.length;
            if (length == 0) {
                continue;
            }
            boolean onlyPrimitives = true;
            for (final Class<?> primitive : parameterTypes) {
                if (primitive.equals(boolean.class) || !Utils.isPrimitiveType(primitive)) {
                    onlyPrimitives = false;
                    break;
                }
            }
            if (!onlyPrimitives) {
                continue;
            }
            
            LOGGER.debug("Testing the method {}", method);
            
            
            final Object[] arrays = new Object[parameterTypes.length];
            for (int i = 0; i < parameterTypes.length; i++) {
                if (int.class.equals(parameterTypes[i])) {
                    arrays[i] = Integer.MAX_VALUE;
                } else if (short.class.equals(parameterTypes[i])) {
                    arrays[i] = Short.MAX_VALUE;
                } else if (long.class.equals(parameterTypes[i])) {
                    arrays[i] = Long.MAX_VALUE;
                } else if (byte.class.equals(parameterTypes[i])) {
                    arrays[i] = Byte.MAX_VALUE;
                } else if (float.class.equals(parameterTypes[i])) {
                    arrays[i] = Float.MAX_VALUE;
                } else if (double.class.equals(parameterTypes[i])) {
                    arrays[i] = Double.MAX_VALUE;
                }
            }
            // MAX VALUE
            this.testMethod(method, arrays);
            // MIN VALUE
            for (int i = 0; i < parameterTypes.length; i++) {
                if (int.class.equals(parameterTypes[i])) {
                    arrays[i] = Integer.MIN_VALUE;
                } else if (short.class.equals(parameterTypes[i])) {
                    arrays[i] = Short.MIN_VALUE;
                } else if (long.class.equals(parameterTypes[i])) {
                    arrays[i] = Long.MIN_VALUE;
                } else if (byte.class.equals(parameterTypes[i])) {
                    arrays[i] = Byte.MIN_VALUE;
                } else if (float.class.equals(parameterTypes[i])) {
                    arrays[i] = Float.MIN_VALUE;
                } else if (double.class.equals(parameterTypes[i])) {
                    arrays[i] = Double.MIN_VALUE;
                }
            }
            
            this.testMethod(method, arrays);
            
        }
        
        
    }
    
    
    /**
     * This method performs the array bounds test
     * 
     * @param method
     *            the method
     * @param arrays
     *            the arrays
     */
    private void testMethod(final Method method, final Object[] arrays) {
    
    
        method.setAccessible(true);
        try {
            method.invoke(this.testedObject, arrays);
            
            
        } catch (final ArrayIndexOutOfBoundsException e) {
            throw new MethodImplementationException(e, method);
        } catch (final Throwable e) {
            LOGGER.error(e.getMessage(), e);
            
        }
        method.setAccessible(false);
    }
    
}
