/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import java.lang.reflect.Method;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.api.IClassTest;




/**
 * This type describes a simple invocation test.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class GetterSetterInvokationTest implements IClassTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(GetterSetterInvokationTest.class);
    private final Object                  testedObject;
    private final int                     excludeMethodsModifiers;
    
    
    
    /**
     * Builds a simple invocation test .
     * 
     * @param _testedObject
     *            the tested class.
     * @param _excludeMethodsModifiers
     */
    public GetterSetterInvokationTest(final Object _testedObject, final int _excludeMethodsModifiers) {
    
    
        super();
        this.testedObject = _testedObject;
        this.excludeMethodsModifiers = _excludeMethodsModifiers;
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.IClassTest#performTest()
     */
    public void performTest() throws WeakImplementationException {
    
    
        LOGGER.debug("Processing invokation of getter / setters");
        for (final Method method : Utils.getMethods(this.testedObject, this.excludeMethodsModifiers)) {
            
            final int length = method.getParameterTypes().length;
            final String methodName = method.getName();
            if ("is".equals(methodName) || "get".equals(methodName) || "set".equals(methodName) && length <= 1) {
                LOGGER.debug("Testing the method {}", method);
                method.setAccessible(true);
                try {
                    if (length == 1) {
                        method.invoke(this.testedObject, new Object[] {
                            null });
                    } else if (length == 0) {
                        method.invoke(this.testedObject);
                    }
                    
                } catch (final Throwable e) {
                    throw new MethodImplementationException(e, method);
                }
                method.setAccessible(false);
            }
        }
        
        
    }
    
}
