/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.api.IClassTest;



/**
 * This type describes a simple invocation test.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class NPEConstructorTest implements IClassTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(NPEConstructorTest.class);
    private final Class<?>                testedClass;
    private final int                     excludeMethodsModifiers;
    
    
    
    /**
     * Builds a simple invocation test .
     * 
     * @param _testedClass
     *            the tested class.
     * @param _excludeMethodsModifiers
     * @param _ignoreImplementations
     */
    public NPEConstructorTest(final Class<?> _testedClass, final int _excludeMethodsModifiers) {
    
    
        super();
        this.testedClass = _testedClass;
        this.excludeMethodsModifiers = _excludeMethodsModifiers;
        
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.IClassTest#performTest()
     */
    public void performTest() throws WeakImplementationException {
    
    
        LOGGER.debug("Processing instantiation of the class {} if any constructors available");
        for (final Constructor<?> constructor : Utils.getConstructors(this.testedClass, this.excludeMethodsModifiers)) {
            
            LOGGER.debug("Testing the constructor {}", constructor);
            constructor.setAccessible(true);
            try {
                final Class<?>[] parameterTypes = constructor.getParameterTypes();
                final Object[] arrays = new Object[parameterTypes.length];
                for (int i = 0; i < parameterTypes.length; ++i) {
                    if (Utils.isPrimitiveType(parameterTypes[i])) {
                        arrays[i] = 0;
                        
                    } else if (boolean.class.isAssignableFrom(parameterTypes[i])) {
                        arrays[i] = false;
                    } else {
                        arrays[i] = null;
                    }
                }
                constructor.newInstance(arrays);
            } catch (final NullPointerException e) {
                throw new ConstructorImplementationException(e, constructor);
            } catch (final InvocationTargetException e) {
                if (e.getCause() instanceof NullPointerException) {
                    throw new ConstructorImplementationException(e, constructor);
                }
            } catch (final Throwable e) {
                //
            }
            constructor.setAccessible(false);
        }
        
        
    }
    
}
