/**
 * Easy Coverage for Java
 * <p>
 * Copyright 2011-2012 Tocea Technology Solutions http://www.tocea.com
 * <p>
 * This library is free software; you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of the License, or (at your option) any later version.
 * <p>
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 * <p>
 * You should have received a copy of the GNU Lesser General Public License along with this library. If not, see
 * <http://www.gnu.org/licenses/>.
 * <p>
 **/

package com.tocea.easycoverage.framework.unit;



import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.slf4j.LoggerFactory;

import com.tocea.easycoverage.framework.behaviour.api.IClassTest;



/**
 * This type describes a simple invocation test.
 * <p>
 * 
 * @author $Author: sleroy $
 * @since 13 mai 2012
 * @copyright Copyright (C) 2010-2012 TOCEA
 */

public class NPEMethodTest implements IClassTest
{
    
    
    //
    // New logger.
    private static final org.slf4j.Logger LOGGER = LoggerFactory.getLogger(NPEMethodTest.class);
    private final Object                  testedObject;
    private final int                     excludeMethodsModifiers;
    
    
    
    /**
     * Builds a simple invocation test .
     * 
     * @param _testedObject
     *            the tested class.
     * @param _excludeMethodsModifiers
     */
    public NPEMethodTest(final Object _testedObject, final int _excludeMethodsModifiers) {
    
    
        super();
        this.testedObject = _testedObject;
        this.excludeMethodsModifiers = _excludeMethodsModifiers;
    }
    
    
    /**
     * (non-Javadoc)
     * 
     * @see com.tocea.easycoverage.framework.behaviour.api.IClassTest#performTest()
     */
    public void performTest() throws WeakImplementationException {
    
    
        LOGGER.debug("Processing instantiation of the class {} if any constructors available");
        for (final Method method : Utils.getMethods(this.testedObject, this.excludeMethodsModifiers)) {
            
            LOGGER.debug("Invokating the method {}", method);
            method.setAccessible(true);
            try {
                final Class<?>[] parameterTypes = method.getParameterTypes();
                final Object[] arrays = new Object[parameterTypes.length];
                for (int i = 0; i < parameterTypes.length; ++i) {
                    if (int.class.equals(parameterTypes[i])) {
                        arrays[i] = 0;
                    } else if (short.class.equals(parameterTypes[i])) {
                        arrays[i] = 0;
                    } else if (long.class.equals(parameterTypes[i])) {
                        arrays[i] = 0;
                    } else if (byte.class.equals(parameterTypes[i])) {
                        arrays[i] = 0;
                    } else if (boolean.class.equals(parameterTypes[i])) {
                        arrays[i] = false;
                    } else if (float.class.equals(parameterTypes[i])) {
                        arrays[i] = 0f;
                    } else if (double.class.equals(parameterTypes[i])) {
                        arrays[i] = 0d;
                    } else {
                        arrays[i] = null;
                    }
                }
                method.invoke(this.testedObject, arrays);
            } catch (final NullPointerException e) {
                throw new MethodImplementationException(e, method);
            } catch (final InvocationTargetException e) {
                if (e.getCause() instanceof NullPointerException) {
                    throw new MethodImplementationException(e, method);
                }
            } catch (final Throwable e) {
                //
            }
            
            
            method.setAccessible(false);
        }
        
        
    }
    
}
